<?php
/*
 *	iptvlist created by Roman Lut aka hax.
 *	
 *	Dionys edit
 *	2016 Dionys, skiner@inbox.ru
 *	2016.10.24
*/

/*
parseEPG() function in this script has to retrive program guide for current day and store it in 
"/tmp/epg_channels.txt" and "/tmp/epg_data.txt", in UTF-8 encoding.
Module is catching parsed results, so script is called only once a day.
Program guide should contain programs starting from 06:00 current date and ending till 06:00 next date - this is important!

epg_channels.txt should contain \n separated lines:
- channel name
- starting position (in bytes) in epg_data.txt
- ending position (in bytes) in epg_data.txt

epg_data.txt should contain \n separated lines:
- start time, in format hour:minute
- program name

All echo output from script is ignored.
There are no error response codes.
If script is unable to retrive EPG due to network error,
it should not create output files.

It is important that timezone is not changed or set to one defined in iptv_timezone when script exits.
*/

header('Content-Type:text/plain');
mb_internal_encoding('UTF-8');

//----------------------------------------------- PHP CUSTOM SETTINGS
set_time_limit(0);
error_reporting(E_ALL ^ E_NOTICE);
ini_set('logdbgs', 'On');
ini_set('error_log', 'parse.error.log');

//----------------------------------------------- DEPENDENCIES
include 'parse_epg_get.php';

//----------------------------------------------- VERSION
$php_ver_array = explode('.', PHP_VERSION);
$php_ver_major = intval($php_ver_array[0]);

//----------------------------------------------- CONSTANTS
define('DEBUG', FALSE);
define('EXEC', FALSE);
define('PHPVER_MAJOR', $php_ver_major);
define('EOS', chr(0x0A));
define('ENT', chr(0x0D) . EOS);
define('DLT', chr(0x09));
define('PROXY', '');
define('URLS', 'parse_epg.cfg');
if (EXEC) {
    define('PATH', './tmp/');
} else {
    define('PATH', '/tmp/');
}
define('CHANNELS', PATH . 'epg_channels.txt');
define('DATA', PATH . 'epg_data.txt');

//----------------------------------------------- VARIABLES
$GLOBALS['codes'] = array('f0', 'd9', 'z0');
$GLOBALS['replaces'] = array('0', '1', '5');
$GLOBALS['urls'] = array('http://www.vsetv.com/schedule_package_tricolor');

//----------------------------------------------- MAIN
if (file_exists('parse_epg_ex.php')) {
    @unlink('parse_epg_ex.php');
}
if (file_exists('parse_epg_lite.php')) {
    @unlink('parse_epg_lite.php');
}
if (is_file(URLS)) {
    $url_list = file(URLS);
    $url_list_count = count($url_list);
    // rebuild update url (skip empty lines)
    for ($i = 0; $i < $url_list_count; $i++) {
        $url_list[$i] = trim($url_list[$i]);
        if (empty($url_list[$i])) {
            unset($url_list[$i]);
        }
    }

    $url_list_count = count($url_list);
    if ($url_list_count > 0) {
        $GLOBALS['urls'] = array_values($url_list);
    }
}
if (EXEC) {
    system('@echo off && title ' . 'TV Parser' . ' (PHP ' . PHPVER_MAJOR . ') && echo ' . 'TV Parser' . ' && echo.');
    print_r($GLOBALS['urls']);
    array_map('unlink', glob(PATH . 'epg_*.*'));
    parseEPG();
    system('echo. && echo     && set /p _NONEED=');
}

//----------------------------------------------- FUNCTIONS
function cut_string_ex($string, $search1, $search2, $offset = 0)
{
    if (gettype($search1) == 'integer') {
        $key1 = $search1;
    } else {
        $key1 = strpos($string, $search1, $offset);
        if ($key1 === FALSE) {
            return FALSE;
        }
        $key1 = $key1 + strlen($search1);
    }

    if (gettype($search2) == 'integer') {
        $key2 = $search2;
    } else {
        $key2 = strpos($string, $search2, $key1);
        if ($key2 === FALSE) {
            return FALSE;
        }
    }

    return array(substr($string, $key1, $key2 - $key1), $key2);
}

function pggcallback($buffer)
{
    return '';
}

function tzDifferent()
{
    // set www.vsetv.com default timezone [(+4) Europe/Moscow]
    date_default_timezone_set('Asia/Dubai');
    $vsetv_time_arr = getdate(time());
    $vsetv_hour = intval($ctime_arr['hours']);

    // set current timezone
    ob_start("pggcallback");
    require 'iptvlist_timezone.php';
    ob_end_flush();
    $curr_time_arr = getdate(time());
    $curr_hour = intval($ctime_arr['hours']);

    return ($curr_hour - $vsetv_hour);
}

function tzCorrection($time, $tzDiff)
{
    $time_arr = explode(':', $time);
    $prHour = intval($time_arr[0]) + $tzDiff;
    $prHourStr = "$prHour";
    $prMinuteStr = $time_arr[1];

    if (strlen($prHourStr) < 2) {
        $prHourStr = '0' . $prHourStr;
    }

    return $prHourStr . ':' . $prMinuteStr;
}

/* Implementatin to retrive program guide from www.vsetv.com */
function doParseEPG($destYear, $destMon, $destDay)
{
    $tzDiff = tzDifferent();
    $ctime_arr = getdate(time());
    $destDay = $ctime_arr['mday'];
    if ($ctime_arr['hours'] < 5) $destDay--;
    if (strlen("$destDay") < 2) $destDay = '0' . $destDay;
    $destMon = $ctime_arr['mon'];
    if (strlen("$destMon") < 2) $destMon = '0' . $destMon;
    $destYear = $ctime_arr['year'];
    $epg_channels = '';
    $epg_data = '';
    $epg_size = 0;
    $channels = array();
    $url_count = count($GLOBALS['urls']);
    $gif_sizes = array();

    for ($i = 0; $i < $url_count; $i++) {
        $url = $GLOBALS['urls'][$i] . '_day_' . $destYear . '-' . $destMon . '-' . $destDay . '_nsc_1.html';
        $html = phpGetPage($url, PROXY);
        $html_size = strlen($html);
        $offset = 0;
        while (true) {
            $title_arr = cut_string_ex($html, '<td nowrap class="channeltitle">', '</td>', $offset);
            if ($title_arr === FALSE) {
                break;
            }
            $offset = $title_arr[1];
            $title = $title_arr[0];
            $key = array_search($title, $channels);
            if ($key !== FALSE && $key !== NULL) {
                continue;
            }
            array_push($channels, $title);
            $epg_channels .= $title . EOS;

            $shedule_array = cut_string_ex($html, '<div id="schedule_container"', "\n", $offset);
            if ($shedule_array === FALSE) {
                break;
            }
            $shedule = $shedule_array[0];
            $shedule = str_replace(array('pasttime', 'onair', 'pastprname2'), array('time', 'time', 'prname2'), $shedule);
            $tmp_data = '';
            $tmp_offset = 0;


            while (true) {
                $time_arr = cut_string_ex($shedule, '<div class="time"' . '>', '</div' . '>', $tmp_offset);
                if ($time_arr === FALSE) {
                    break;
                }
                $tmp_offset = $time_arr[1];
                $time = str_replace(array('<a class=', '></a>'), '', $time_arr[0]);
                $time = str_replace($GLOBALS['codes'], $GLOBALS['replaces'], $time);

/* Repalce images 0, 5 to text */
                if (strpos($time, ".gif")) {

                    $dom = new DOMDocument;
                    $dom->loadHTML($time);
                    $images = $dom->getElementsByTagName('img');


                    foreach ($images as $image) {
                        $gif_path = $image->getAttribute("src");

                        if (array_key_exists($gif_path, $gif_sizes)) {
                            $pic_length = $gif_sizes[$gif_path];
                        } else {
                            $headers = get_headers('http://www.vsetv.com' . $gif_path, true);
                            $pic_length = $headers["Content-Length"];
                            $gif_sizes[$gif_path] = $pic_length;
                        }


                        if ($pic_length == 1213) {
                            $time = str_replace('<img src="' . $gif_path . '">', '0', $time);
                            $time = str_replace('<img src="' . $gif_path . '">', '0', $time);
                        }

                        if ($pic_length == 1217) {
                            $time = str_replace('<img src="' . $gif_path . '">', '5', $time);
                            $time = str_replace('<img src="' . $gif_path . '">', '5', $time);
                        }


                    }
                }

                $time = tzCorrection($time, $tzDiff);
                $tmp_data .= $time . EOS;

                $name_arr = cut_string_ex($shedule, '<div class="prname2"' . '>', '</div' . '>', $tmp_offset);
                if ($time_arr === FALSE) {
                    break;
                }
                $tmp_offset = $name_arr[1];
                $name = $name_arr[0];
                $name = html_entity_decode($name, ENT_QUOTES, 'windows-1251');
                $name = str_replace("\xa0", '', $name);
                $name = str_replace("\n", '', $name);
                $name = str_replace("\r", '', $name);
                $name = trim($name);
                $name = strip_tags($name);
                $tmp_data .= $name . EOS;
            }

            $epg_channels .= $epg_size . EOS;
            $tmp_data = iconv("windows-1251", "utf-8", $tmp_data);
            $epg_size += strlen($tmp_data);
            $epg_channels .= $epg_size . EOS;
            $epg_data .= $tmp_data;
        }

        //  2016.10.23  jquery-gs.js
        if (DEBUG && $i == 0) {
            $json = phpGetPage('http://www.vsetv.com/jquery-gs.js', PROXY);
            $json_f = cut_string_ex($html, 'ready(function(){', '();');
            $json_key = strpos($json, $json_f[0]);
            $time_codes_url = cut_string_ex($json, 'url: "/', '"', $json_key);
            $time_codes = phpGetPageEx('http://www.vsetv.com/' . $time_codes_url[0], PROXY, 'html', '', 'X-Requested-With: XMLHttpRequest');
            $time_codes_array = cut_string_ex($time_codes, '{', '}');
            $time_codes = str_replace(array('"', '.'), '', $time_codes_array[0]);
            $time_replace_array = cut_string_ex($json, 'JSON.parse(data);', '}', $json_key);
            $time_replace = str_replace(DLT, '', $time_replace_array[0]);
            file_put_contents(PATH . 'epg_codes.txt', $time_codes . ENT . $time_replace . ENT);
        }

        if (DEBUG) {
            file_put_contents(PATH . 'epg_' . $i . '.html', $html);
        }
    }

    if (!empty($epg_channels) && !empty($epg_data)) {
        $epg_channels = iconv("windows-1251", "utf-8", $epg_channels);

        file_put_contents(CHANNELS, $epg_channels);
        file_put_contents(DATA, $epg_data);
    }
}

/* only calls actual parser if cache is outdated or does not exist */
function parseEPG()
{
    ob_start('pggcallback');
    require 'iptvlist_timezone.php';
    ob_end_flush();

    $ctime_arr = getdate(time());
    $focusHour = $ctime_arr['hours'];

    $ctime_day = $ctime_arr['mday'];
    if ($ctime_arr['hours'] < 5) {
        $ctime_day--;
        $focusHour += 24;
    }

    if (!file_exists(CHANNELS)) {
        doParseEPG($ctime_arr['year'], $ctime_arr['mon'], $ctime_day);
    } else {
        //check if EPG still valid
        $ftime = filemtime(CHANNELS);
        $ftime_arr = getdate($ftime);

        $ftime_day = $ftime_arr['mday'];
        if ($ftime_arr['hours'] < 5) {
            $ftime_day--;
        }

        if ($ftime_arr['year'] != $ctime_arr['year'] || $ftime_arr['mon'] != $ctime_arr['mon'] || $ftime_day != $ctime_day) {
            doParseEPG($ctime_arr['year'], $ctime_arr['mon'], $ctime_day);
        }
    }
}

?>